"use strict";var r=require("../register-BOkp8V6j.cjs");require("../get-pipe-path-D4YM6rQt.cjs"),require("module"),require("node:path"),require("../temporary-directory-B83uKxJF.cjs"),require("node:os"),require("node:module"),require("node:url"),require("node:fs"),require("fs"),require("os"),require("path"),require("../index-D9F1FXzN.cjs"),require("esbuild"),require("node:crypto"),require("../node-features-CQLdkVE6.cjs"),require("../client-D3mGB526.cjs"),require("node:net"),require("node:util"),require("../index-BWFBUo6r.cjs"),r.register();
