/*!
  Copyright 2013 Lovell Fuller and others.
  SPDX-License-Identifier: Apache-2.0
*/

const Sharp = require('./constructor.cjs');
const input = require('./input.cjs');
const resize = require('./resize.cjs');
const composite = require('./composite.cjs');
const operation = require('./operation.cjs');
const colour = require('./colour.cjs');
const channel = require('./channel.cjs');
const output = require('./output.cjs');
const utility = require('./utility.cjs');

input(Sharp);
resize(Sharp);
composite(Sharp);
operation(Sharp);
colour(Sharp);
channel(Sharp);
output(Sharp);
utility(Sharp);

module.exports = Sharp;
