export type CrossEnvOptions = {
    shell?: boolean;
};
export type ProcessResult = {
    exitCode: number | null;
    signal?: string | null;
};
export declare function crossEnv(args: string[], options?: CrossEnvOptions): ProcessResult | null;
