import { type Connect, type Logger } from 'vite';
interface Config {
    readonly debugLogger: Logger;
}
export default function middlewareAccessLog({ debugLogger }: Config): Connect.NextHandleFunction;
export {};
