import type { IncomingMessage, ServerResponse } from 'node:http';
import type { Connect } from 'vite';
export declare function createMiddleware(middleware: (request: IncomingMessage, response: ServerResponse) => void | Promise<void>): Connect.NextHandleFunction;
