import { type Connect, type Logger } from 'vite';
interface Config {
    readonly debugLogger: Logger;
    readonly getBuildError: () => Error | undefined;
}
/**
 * Middleware that serves an error page when an error is present.
 */
export default function middlewareBuildError({ debugLogger, getBuildError }: Config): Connect.NextHandleFunction;
export {};
