import "vite";
import { createMiddleware } from "../util/create-middleware.js";
function middlewareAccessLog({ debugLogger }) {
  return createMiddleware((req, res) => {
    const start = Date.now();
    res.on("finish", () => debugLogger.info(`${req.method} ${req.url} ${res.statusCode} ${Date.now() - start}ms`));
  });
}
export {
  middlewareAccessLog as default
};
//# sourceMappingURL=access-log.js.map
