import { ValidationOptions } from '../ValidationOptions';
/**
 * If object has both allowed and not allowed properties a validation error will be thrown.
 */
export declare function Allow(validationOptions?: ValidationOptions): PropertyDecorator;
