import type { Mutex } from '@seahax/semaphore';
import type { Connect } from 'vite';
export interface Config {
    readonly mutex: Mutex<'build' | 'preview'>;
}
export default function middlewareMutex({ mutex }: Config): Connect.NextHandleFunction;
